*! version 2.1.4  16mar1998
program define bstrap
	version 5.0

/* Parse. */

	parse "`*'", parse(" ,")
	local prog "`1'"
	if "`prog'"=="" | "`prog'"=="," {
		di in red "program name required"
		exit 198
	}
	mac shift

	#delimit ;
	local options "ARgs(str) Reps(int 50) SIze(int -9) Dots
		CLuster(str) IDcluster(str) Level(real $S_level)
		SAving(str) DOUBle EVery(int -9) REPLACE
		noADjust LEAVE" ;
		/* noADjust included for compatibility with version 4.0 */
	#delimit cr

	parse "`*'"

/* Check options. */

	if `size' == -9 { local size }
	else if `size' < 1 {
		di in red "size() invalid"
		exit 198
	}
	if "`cluster'"!="" {
		unabbrev `cluster'
		local cluster "$S_1"
	}
	if "`idclust'"!="" & "`cluster'"=="" {
		di in red "idcluster() can only be specified with " /*
		*/ "cluster() option"
		exit 198
	}
	if "`dots'"=="" {
		local dots "*"
	}
	else	local dots "noisily"

	if `every' == -9 { local every }
	else local every "every(`every')"

	if "`saving'"=="" {
		tempfile saving
		local filetmp "yes"
        	if "`every'"!="" {
                	di in red "every() can only be specified when " /*
			*/ "using saving() option"
                	exit 198
        	}
        	if "`replace'"!="" {
                	di in red "replace can only be specified when " /*
			*/ "using saving() option"
                	exit 198
        	}
	}

	if `level' <= 0 | `level' >= 100 {
		di in red "level() must be between 0 and 100"
		exit 198
	}

/* Initialization.  User program is called before preserve in case initializer
   wants to load some data set.
*/
	if "`prog'"!="_BS" {
		global S_1
		capture `prog' ? `args'
		if _rc {
			if _rc==199 {
				di in red "program `prog' not found or"
				di in red _quote "`prog' ?" _quote /*
				*/ " attempted to execute an unrecognized" /*
				*/ " command"
				exit 199
			}
			di in red _quote "`prog' ?" _quote " returned:"
			error _rc
		}
		if "$S_1"=="" {
			di in red _quote "`prog' ?" _quote " did not set \$S_1"
			exit 198
		}
		local vl "$S_1"
	}
	else	local vl "$S_bs_vl"

/* If clusters, count number.  Display #clusters or #obs and check size. */

	if "`cluster'"!="" {
		tempvar flag
		sort `cluster'
		quietly by `cluster': gen byte `flag' = (_n==1)
		quietly count if `flag'
		local nclust = _result(1)
		di in gr "(clusters=" `nclust' ")"
		if "`size'"!="" {
			if `size' > `nclust' {
				di in red "size() must not be greater than " /*
				*/ "number of clusters"
				exit 498
			}
		}
	}
	else {
		di in gr "(obs=" _N ")"
		if "`size'"!="" {
			if `size' > _N {
				di in red "size() must not be greater than " /*
				*/ "number of observations"
				exit 498
			}
			else if `size' == _N { local size }
				/* bsample prefers call without arguments */
		}
	}

/* Preserve and initialize post. */

	preserve
	tempname postnam

	postfile `postnam' `vl' using "`saving'", `double' `every' `replace'

/* Do reps. */

	quietly {
		if "`idclust'"!="" {
			capture confirm new variable `idclust'
			if _rc == 0 {
				replace `flag' = sum(`flag')
				label variable `flag' /*
				*/ "Bootstrap sample cluster id"
				rename `flag' `idclust'
			}
			else confirm variable `idclust'

			local idopt "id(`idclust')"
		}

		`prog' `postnam' `args' /* first call with dataset as is */

		local i 1
		while `i' <= `reps' {
			`dots' di in gr "." _c
			restore, preserve

			if "`cluster'"=="" {
				bsample `size'
			}
			else {
				bsample `size', cluster(`cluster') `idopt'
			}
			`prog' `postnam' `args'
			local i = `i' + 1
		}
		`dots' di
		postclose `postnam'

/* Load file `saving' with bootstrap results. */

		capture use "`saving'", clear
		if _rc {
			if _rc >= 900 & _rc <= 903 {
				di in red "insufficient memory to load " /*
				*/ "file with bootstrap results"
			}
			error _rc
		}

		if "`prog'"=="_BS" {
			Labdat bs: $S_bs_cmd
			* label data "bs: $S_bs_cmd"
		}
		else {
			if "`cluster'"=="" { label data "`prog' bootstrap" }
			else { label data "`prog' clustered bootstrap" }
		}

		parse "`vl'", parse(" ")
		local i 1
		while "``i''"!="" {
			local x = ``i''[1]
			char ``i''[bstrap] `x'
			if "`prog'"=="_BS" & "`filetmp'"=="" {
				local varlab : word `i' of $S_bs_lab
				label variable ``i'' "`varlab'"
			}
			local i = `i' + 1
		}
		drop in 1

		if "`filetmp'"=="" { quietly save "`saving'", replace }
	}
	if "`leave'"!="" {
		global S_FN
		restore, not
	}

	bstat, level(`level')
end

program define _BS	/* program called by bs.ado */
	version 5.0

	$S_bs_cmd /* run command */

	SubMacro $S_bs_st /* substitute values for any S_# macros */

	post `1' $S_1
end

program define SubMacro
	version 5.0
	global S_bs_so /* erase macro */
	local k 1
	while "``k''"!="" {
		local i = index("``k''","S_")
		local n = length("``k''")
		local j 1
		while `i' != 0 & `j' <= `n' {
			local front = substr("``k''",1,`i'-1)
			local back  = substr("``k''",`i',.)
			local `k' "`front'$`back'"
			local i = index("``k''","S_")
			local j = `j' + 1 /* prevents infinite loop if error */
		}
		local stats "`stats' ``k''"
		local k = `k' + 1
	}
	global S_1 "`stats'"
end

program define Labdat
	local x = trim("`1' `2' `3' `4' `5' `6' `7'")
	label data "`x'"
end
