*! version 2.1.0 PR 11Dec97.
program define fracplot
version 5.0
local FP= ("$S_E_fp"=="fracpoly")
if "$S_E_fp"!="mfracpol" & !`FP' { error 301 }
local cmd "$S_E_cmd"
if "`cmd'"=="mlogit" | $S_E_bloc | ${S_E_dist}==4 {
	di in bl _n "(graph not supported for `cmd')"
	exit
}
local dist $S_E_dist
local y "$S_E_depv"
local if "opt"
local in "opt"
local hidopts "DResid(string) ETa(string) noGRaph noPTs noWTs"
if `FP' { local hidopts "`hidopts' BAse" }
local options "Symbol(string) Connect(string) T1title(string) `hidopts' *"
/*
	Set things up for a parse of `*'
*/
if `FP' {
	parse "`*'", parse(" ,")
	if !("`1'"==""|"`1'"=="if"|"`1'"=="in"|"`1'"==",") {
		local varlist "opt max(1)"
	}
}
else {
	local varlist "req max(2) min(1)"
}
parse "`*'"
if "`pts'"=="nopts" & "`dresid'"!="" {
	di in red "cannot calculate residuals if nopts specified"
	exit 198
}
/*
	Ensure that we can plot a fracpoly base var as a covariate
*/
if "`base'"!="" { local FP 0 }
if `FP' {
	if "`varlist'"!="" {
		local x `varlist'
	}
	else local x "$S_E_rhs"		/* X-axis variable for plot */
	local xp "$S_E_xp"
	local base "$S_E_base"
	local powers $S_E_pwrs
}
else {
	parse "`varlist'",  parse(" ")
	local rhs `1'
	local x `2'			/* X-axis variable for plot */
	if "`x'"=="" { local x `rhs' }
	local i 0
	while `i'<$S_E_nx {
		local i=`i'+1
		if "`rhs'"=="${S_E_x`i'}" & "${S_E_k`i'}"!="." {
			local num `i'
			local i $S_E_nx
		}
	}
	if "`num'"=="" {
		di in red "variable `rhs' not in final model"
		exit 198	
	}
	local len=min(6,length("`rhs'"))
	local nx: word count $S_E_fvl
	local base
	local xp
	local i 0
	while `i'<`nx' {
		local i=`i'+1
		local v: word `i' of $S_E_fvl
		if substr("`v'",1,`len')!=substr("${S_E_x`num'}",1,`len') {
			local base `base' `v'
		}
		else local xp `xp' `v'
	}
	local powers ${S_E_k`num'}
}
global S_E_vl $S_E_fvl
if "$S_E_wgt"!="" & "`wts'"!="nowts" {
	local exp "$S_E_exp"	/* for weighted residual calculation */
*	local wtplot "[$S_E_wgt $S_E_exp]" /* cd be used for weighted plot */
}
tempvar etahat
predict `etahat' if _sample, xb
quietly {
	if "`pts'"=="" {	/* points to be plotted */
		tempvar yv dr
		gen `yv'=`y' if _sample
		gen `dr'=.
		residual `dist' `yv' `etahat' `dr' "`exp'"
	}
	sum `etahat'
	local ebar = _result(3)
	if `dist'==3 {	/* centre Cox linear predictor on its mean */
		replace `etahat'=`etahat'-`ebar'
		local ebar 0
	}
	if "`base'"!="" {
		tempvar cmpnt
		local adj ", adjusted for covariates"
		parse "`xp'",parse(" ")
		gen `cmpnt' = _b[`1']*`1'
		mac shift
		while "`1'"!=""{
			replace `cmpnt'=`cmpnt'+_b[`1']*`1'
			mac shift
		}
		sum `cmpnt', meanonly
		replace `etahat' = `cmpnt'+`ebar'-_result(3)
	}
}
if "`t1title'"=="" {
	_fracmdp 4 `powers'
	local t1title "$S_E_t1t ($S_1)`adj'"
}
lab var `etahat' "Fitted component"
if "`pts'"=="" {
	qui replace `yv'=`etahat'+`dr'
	lab var `yv' "Component+residual for `y'"
	if "`symbol'"=="" { local symbol "oi" }
	if "`connect'"=="" { local connect ".s" }
}
else {
	if "`symbol'"=="" { local symbol "i" }
	if "`connect'"=="" { local connect "s" }
}
/*
	Component (-plus partial-residual) plot
*/
if "`graph'"=="" {
	graph `yv' `etahat' `x' `if' `in', t1title("`t1title'") /*
	 */ s(`symbol') c(`connect') `options'
}
if "`eta'"!="" {
	confirm new var `eta'
	rename `etahat' `eta'
}
if "`dresid'"!="" {
	confirm new var `dresid'
	rename `dr' `dresid'
}
end

program define residual /* deviance or martingale residuals */
	local dist `1'
	local y `2'
	local eta `3'
	local r `4'
	local exp "`5'"
	if `dist'==0 { /* normal */
		replace `r' = `y'-`eta'
	}
	else if `dist'==1 { /* logit; does not work for probit */
		local cmd $S_E_cmd
		global S_E_vl "`y' $S_E_vl"
		global S_E_cmd "logistic"
		cap drop `r'
		lpredict `r', deviance
		replace `r'=. if _sample==0	/* !! redundant? */
		global S_E_vl
		global S_E_cmd `cmd'
	}
	else if `dist'==2 { /* Poisson */
		tempvar mu
		gen `mu' = exp(`eta')
		replace `r' = sign(`y'-`mu') /*
		 */ *sqrt(cond(`y'==0, 2*`mu', /*
		 */ 2*(`y'*log(`y'/`mu')-(`y'-`mu'))))
	}
	else if `dist'==3 {	/* refit Cox model on linear predictor only */
		local wgt [$S_E_wgt $S_E_exp]
		local opts $S_E_opts
		tempname ests
		est hold `ests'
		cap drop `r'
		cox `y' `eta' `wgt', `opts' mgale(`r')
		est unhold `ests'
	}
	if "`exp'"!="" {
		parse "`exp'", parse("=")
/*
	Weighted residuals using weights standardized Stata-fashion
*/
		sum `2' if _sample, meanonly
		replace `r' = `r'*sqrt(`2'/_result(3))
	}
end
