*! version 1.2.2  16jun1998                                    (STB-45 release)
program define heckman
	version 4.0
	local options "Level(integer $S_level)"
	if "`1'"!="" & substr("`1'",1,1)!="," {
		parse "`*'", parse(" ,")
		local regeq `1'
		local probit `2'
		mac shift 2

		local options /*
		*/ "`options' noConstant DACC(real 1e-5) HOLDrho noLOg TRace *"
		local in "opt"
		local if "opt"
		local weight "fweight aweight"
		parse "`*'"

		if "`log'"=="" { local log "noisily" }
		else local log 

		if "`trace'"!="" { local noi "noisily" }

		eq ? `regeq'
		local regeq $S_3
		parse "$S_1", parse(" ")
		local dv `1'
		if "`dv'"=="" { local dv $S_3 }
		mac shift
		local vreg "`*'"

		eq ? `probit'
		local probit $S_3
		local vprobit "$S_1"

		tempvar touse touse2 mysamp isobs pxb mills
		tempname pb rb sigma rho f V bb
		mark `touse' [`weight'`exp'] `if' `in'
		markout `touse' `vprobit' 
		mark `touse2'
		markout `touse2' `vreg' /* only matters if `dv'!=. */

		quietly {
			replace `touse' = 0 if `touse2'==0 & `dv'!=.

		/* Check that `dv' has missing values. */

			count if `dv'==. & `touse'
			if _result(1)==0 { 
				di in red "`dv' is never missing"
				exit 499
			}
	
		/* Do probit. */
			gen byte `isobs' = (`dv'!=.) if `touse'
			qui _rmcoll `vprobit'
			local vprobit $S_1
			local kprobit : word count `vprobit'
			local kprobit = `kprobit' + ( "`constan'" == "" )
			`noi' probit `isobs' `vprobit' [`weight'`exp'] /*
				*/	if `touse', `constan' nolog

			mat `pb' = get(_b)
			local prbcol : colnames(`pb')
			local nprobit : word count `prbcol'
			if `nprobit' < `kprobit' {
				/* special probit if perfect pred */
				noi di in green "Computing starting values."
				`noi' heckpbt `isobs' `vprobit' [`weight'`exp'] /*
					*/	if `touse', `constan' nolog
				mat `pb' = get(_b)
				noi di in green "Fitting full model:"
			}

			local df = _result(3)
			predict `pxb' if `touse', index
			mat coleq `pb' = `probit'

		/* Compute Mills ratio. */

			gen `mills' = exp(-0.5*`pxb'^2) /*
			*/	      /(sqrt(2*_pi)*normprob(`pxb')) if `touse'

		/* Do initial regression with Mills' ratio. */

			`noi' di _n in gr "Mills' ratio coefficient " /*
			*/	          "estimate from regression"

			`noi' reg `dv' `mills' `vreg' [`weight'`exp'] /*
			*/	if `touse', `constan'

			`noi' di _n in gr "Coefficient of Mills' ratio " /*
			*/ in ye %9.0g _b[`mills']

			local df = `df' + _result(3) - ("`holdrho'"!="")
			scalar `sigma' = _result(9)

			scalar `rho' = _b[`mills']/`sigma'

			if "`trace'"!="" | abs(`rho')>0.9 {
				noi di in gr "Initial estimate of rho     " /*
				*/     in ye %9.0g `rho'
			}
			if abs(`rho')>0.9 { 
				if abs(`rho')>0.99 {
					scalar `rho' = sign(`rho')*0.99
					noi di in gr "Initial rho set to " /*
					*/ in ye %14.2f `rho'
				}
				#delimit ;
				noi di in blu
                                   "Heckman model may be inappropriate "
				   "since initial |rho| > 0.9" ;
				#delimit cr
			}
			`noi' di _n in gr "ML computation begins ... "

		/* Set up vector of initial parameter estimates. */

			mat `bb' = J(1,2,0)
			mat colnames `bb' = _cons _cons
			mat coleq `bb' = _athrho _lnsigma

		/* Transform `rho' using atanh (inverse hyperbolic tangent). */

			mat `bb'[1,1] = 0.5*log((1+`rho')/(1-`rho'))

			if "`holdrho'"!="" { global S_mlhr = `rho' }
			else global S_mlhr

			mat `bb'[1,2] = ln(`sigma')

			mat `rb' = get(_b)
			mat `rb' = `rb'[1,2...]
			mat coleq `rb' = `regeq'
			mat `rb' = `rb' , `pb'
			mat `rb' = `rb' , `bb'

		/* Compute lf0. */
			summarize `isobs' [`weight'`exp'] if `touse'
			local lf0 = _result(1)*(_result(3)*ln(_result(3)) /* 
				*/ + (1-_result(3))*ln(1-_result(3)))
			tempname wtobs
			if "`weight'" == "aweight" {
				tempvar wt
				gen double `wt' `exp'  if `touse'
				sum `wt', meanonly
				replace `wt' = `wt' * _result(1) / _result(18)
				sum `wt', meanonly,  if `touse' & `isobs'
				scalar `wtobs' = _result(18)
				reg `dv' `vreg' [iweight=`wt'] if `touse', `constan'
			}
			else {
				reg `dv' `vreg' [`weight'`exp'] if `touse', `constan'
				scalar `wtobs' = _result(1)
			}
	
			local lf0 = `lf0' +  (-0.5 * `wtobs' * /*
				*/  ( ln(2*_pi) + ln(_result(4) / `wtobs') + 1 ) )
		}

	/* Sort.  ml_heck.ado requires the last obs to be nonmissing. */

		sort `touse'

	/* Run ml commands. */

		ml begin
		global S_mleqn "`reqeq' `probit' _athrho _lnsigma"
		ml func ml_heck
		ml method lf
		ml sample `mysamp' [`weight'`exp'] if `touse', noauto
		ml depn `dv' `isobs'

		global S_mlmdf "`df'"
		global S_mlmb  "`rb'"

		cap `log' ml max `f' `V', `options' `trace'

		local rc = _rc
		if `rc'>1 {
			di in red "(error occurred in ML computation)"
			di in red "(use trace option and check correctness " /*
			*/ "of initial model)" 
			error `rc'
		}
		if `rc'==1 {
			di in blu /*
			*/ "(Maximization aborted, computing standard errors)"
			ml report
			exit
		}
		ml depn `dv'
		ml post heckman, title(Heckman selection model) lf0(`lf0')
	}
	else {
		if "$S_E_cmd"!="heckman" {error 301}
		parse "`*'"
	}
	if `level'<10 | `level'>99 { local level 95 }
	ml mlout heckman, level(`level')
	heckrpt
	#delimit ;
	di in gr "     rho   " in ye %10.5f $S_1
		in gr _col(38) "[_athrho]_cons  = atanh(rho)" ;
	di in gr "   sigma   " in ye %10.0g $S_2
		in gr _col(38) "[_lnsigma]_cons = ln(sigma)" ;
	#delimit cr
	di in gr "  lambda   " in ye %10.0g $S_3 "  "  %9.0g $S_4
end


program define heckrpt
	version 4.0
        tempname v arho rho sigma lambda d
        mat `v' = get(VCE)
        scalar `arho'   = [_athrho]_cons
        scalar `rho'    = (exp(2*`arho')-1)/(exp(2*`arho')+1)
        scalar `sigma'  = exp([_lnsigma]_cons)
        scalar `lambda' = `rho'*`sigma'
        mat `d' = J(1,2,0)
        mat `d'[1,1] = `sigma'*4*exp(2*`arho')/(exp(2*`arho')+1)^2
        mat `d'[1,2] = `lambda'
        mat `v' = `v'["_athrho:_cons".."_lnsigma:_cons", /*
        */            "_athrho:_cons".."_lnsigma:_cons"]
        mat `v' = `d'*`v'
        mat `v' = `v'*`d''
	global S_1 = `rho'
	global S_2 = `sigma'
	global S_3 = `lambda' 
	global S_4 = sqrt(`v'[1,1])
end
