*! version 1.0.4  23nov1998
program define svy_olog
	version 5.0
	local cmd "`1'"
	macro shift
	if substr("`1'",1,1)=="," | "`*'"=="" {
		if !(("`cmd'"=="ologit"  & "$S_E_cmd"=="svyolog") /*
		*/  |("`cmd'"=="oprobit" & "$S_E_cmd"=="svyoprob")) {
			error 301
		}
		svy_dreg `*' /* display results */
		exit
	}
	capture noisily {
		tempvar doit
		tempname s b V cuts Vdeff

		svy_preg `cmd' `doit' `*'

	/* Parse options. */

		local options "LOg TRace *"
		parse ", $S_VYeopt"
		if "`log'`trace'"=="" { local quiet "quietly" }
		local eopt "`trace' `options'"
		local options "MEFF MEFT *"
		parse ", $S_VYdopt"

	/* Ready weights for estimation. */

		if "$S_VYwgt"!="" {
			local awt "[aw=$S_VYexp]"
			local wt  "[iw=$S_VYexp]"
			local cwt "& ($S_VYexp)!=0"
		}

	/* Get ologit/oprobit estimates. */

		`quiet' di _n in gr "Computing weighted estimates ..."
		`quiet' `cmd' $S_VYdepv $S_VYindv `awt' if $S_VYsub, `eopt'

		if _result(1)==0 | _result(1)==. {
			error 2000
		}
		qui count if $S_VYsub `cwt'  /* can't trust ologit/oprobit
						to count #obs
					     */
		local nobs = _result(1)
		matrix `b' = get(_b)
		matrix `V' = get(VCE)

	/* Get number of cutpoints. */

		local names : colnames(`b')
		parse "`names'", parse(" ")
		local dim = colsof(`b')
		local ncut = substr("``dim''",5,.)
		global S_VYncat = `ncut' + 1 /* number of categories */

	/* Use colnames of `b' for indepvars. */

		local start = `dim' - `ncut' + 1 /* position of 1st _cut */
		local `start' /* truncate `*' to remove _cut's */
		global S_VYindv "`*'"
		local mdf = `start' - 1

	/* Get tempvars to hold scores and build up equation names. */

		if `mdf' > 0 {
			tempvar tmp
			local scores "`tmp'"
		}
		local names /* erase macro */
		local i 1
		while `i' <= `ncut' {
			tempvar tmp
			local scores "`scores' `tmp'"
			local names "`names' _cut`i'"
			local i = `i' + 1
		}

	/* Relabel `b' and `V'. */

		matrix `cuts' = `b'[1,`start'...]
		matrix coleq    `cuts' = `names'
		matrix colnames `cuts' = _cons

		if `mdf' > 0 {
			matrix `b' = `b'[1,1..`mdf']
			matrix coleq `b' = $S_VYdepv
			matrix `b' = `b' , `cuts'
		}
		else	matrix `b' = `cuts'

		local names : colnames(`b')
		matrix colnames `V' = `names'
		matrix rownames `V' = `names'
		local names : coleq(`b')
		matrix coleq `V' = `names'
		matrix roweq `V' = `names'

	/* Compute scores. */

		Scores `cmd' $S_VYdepv $S_VYsub `b' `scores'

	/* Unwind weight scaling of ologit/oprobit. */

		if "$S_VYwgt"!="" {
			tempvar sw
			qui gen double `sw' = sum($S_VYsub*($S_VYexp))
			scalar `s' = `nobs'/`sw'[_N]
			matrix `V' = `s'*`V'
		}

	/* Get ologit/oprobit estimates for meff. */

		if "`meff'`meft'"!="" {
			tempname Vmeff
			`quiet' di _n in gr "Computing unweighted estimates" /*
			*/ " for meff and meft ..."
			`quiet' `cmd' $S_VYdepv $S_VYindv if $S_VYsub, `eopt'
			if _result(1)!=`nobs' {
				matrix `Vmeff' = J(`dim',`dim',0)
			}
			else {
				matrix `Vmeff' = get(VCE)
				local dimMeff = colsof(`Vmeff')
				if `dimMeff'!=`dim' {
					matrix `Vmeff' = J(`dim',`dim',0)
				}
			}
		}

	/* Compute variance estimate. */

		_robust `scores' `wt' if `doit', v(`V') vsrs(`Vdeff') /*
		*/	$S_VYopt zeroweight

	/* Post results. */

		local df = _result(3) - _result(2)

		matrix post `b' `V', dof(`df') depn($S_VYdepv)

	/* Set macros, etc. */

		local doption "$S_VYdopt" /* save display options */

		svy_preg save `Vdeff' `Vmeff' /* macros, matrices, F test */

		if "`cmd'"=="ologit" { global S_E_cmd "svyolog" }
		else global S_E_cmd "svyoprob"
	}
	nobreak {
		if _rc {
			local rc = _rc
			macro drop S_VY*
			discard
			exit `rc'
		}
	}

	svy_dreg, `doption'  /* display results */
end

program define Scores /* cmd depvar markvar beta <variables for scores> */
	local cmd "`1'"
	local y   "`2'"
	local sub "`3'"
	local b   "`4'"
	macro shift 4

	tempvar xb iy
	tempname C

	quietly {

	/* Get categories. */

		tab `y' if `sub', matrow(`C')
		local nc = _result(2)

	/* Create `iy' = i if `y' = i-th category. */

		gen byte `iy' = . in 1
		local i 1
		while `i' <= `nc' {
			replace `iy' = `i' if `y'==`C'[`i',1] & `sub'
			local i = `i' + 1
		}

		local start = colsof(`b') - `nc' + 2
		matrix `C' = `b'[1,`start'...] /* cutpoints */

		if "$S_VYindv"!="" { predict double `xb' }
		else local xb 0

		local cmd = substr("`cmd'",1,2)

		Score_`cmd' `iy' `xb' `C' `nc' `*'
	}
end

program define Score_ol
	local iy   "`1'"
	local xb   "`2'"
	local C    "`3'"
	local nc   "`4'"
	macro shift 4

	#delimit ;
	if "$S_VYindv"!="" { ;
		gen double `1' =
			cond(`iy'==1,
			     -exp(`xb'-`C'[1,1])/(1+exp(`xb'-`C'[1,1]))
			     ,
			     cond(`iy'==`nc',
				  exp(`C'[1,`nc'-1]-`xb')
				  /(1+exp(`C'[1,`nc'-1]-`xb'))
				  ,
				  cond(`iy'!=.,
				       (1-exp(2*`xb'-`C'[1,`iy']-`C'[1,`iy'-1])
				       )
				       /((1+exp(`xb'-`C'[1,`iy']))
				       *(1+exp(`xb'-`C'[1,`iy'-1])))
				       ,
				       0
				  )
			     )
			) ;
		macro shift ;
	} ;
	local i 1 ;
	while `i' <= `nc' - 1 { ;
		gen double ``i'' =
			cond(`iy'==`i',
			     cond(`i'==1,
			          exp(`xb'-`C'[1,1])
				  ,
			          (1+exp(`xb'-`C'[1,`i'-1]))
			          /(exp(`C'[1,`i']-`C'[1,`i'-1])-1)
			     )
			     ,
			     cond(`iy'==`i'+1,
				  cond(`i'+1==`nc',
				       -1
				       ,
			               (1+exp(`xb'-`C'[1,`i'+1]))
			               /(exp(`C'[1,`i']-`C'[1,`i'+1])-1)
			          )
				  ,
			          0
			     )
			)
			/(1+exp(`xb'-`C'[1,`i'])) ;

		local i = `i' + 1 ;
	} ;
	#delimit cr
end

program define Score_op
	local iy   "`1'"
	local xb   "`2'"
	local C    "`3'"
	local nc   "`4'"
	macro shift 4

	#delimit ;
	if "$S_VYindv"!="" { ;
		gen double `1' =
			cond(`iy'==1,
			     -normd(`C'[1,1]-`xb')/normprob(`C'[1,1]-`xb')
			     ,
			     cond(`iy'==`nc',
				  normd(`C'[1,`nc'-1]-`xb')
				  /(1-normprob(`C'[1,`nc'-1]-`xb'))
				  ,
				  cond(`iy'!=.,
				       (-normd(`C'[1,`iy']-`xb')
				       +normd(`C'[1,`iy'-1]-`xb'))
				       /(normprob(`C'[1,`iy'  ]-`xb')
					-normprob(`C'[1,`iy'-1]-`xb'))
				       ,
				       0
				  )
			     )
			) ;
		macro shift ;
	} ;

	local i 1 ;
	while `i' <= `nc' - 1 { ;
		gen double ``i'' =
			cond(`iy'!=`i'&`iy'!=`i'+1,
                             0
			     ,
			     normd(`C'[1,`i']-`xb')
			     /( normprob(`C'[1,`i']-`xb')
			        + cond(`iy'==`i',
			               cond(`i'==1,
                                            0
                                            ,
				            -normprob(`C'[1,`i'-1]-`xb')
				       )
				       , /* `iy'==`i'+1 */
				       cond(`i'+1==`nc',
				            -1
				            ,
			                    -normprob(`C'[1,`i'+1]-`xb')
				       )
			          )
			     )
			) ;

		local i = `i' + 1 ;
	} ;
	#delimit cr
end
