*! version 1.0.2  23nov1998
program define svymlog
	version 5.0
	if substr("`1'",1,1)=="," | "`*'"=="" {
		if "$S_E_cmd"!="svymlog" {
			error 301
		}
		svy_dreg `*' /* display results */
		exit
	}
	capture noisily {
		tempvar doit
		tempname b V A C s Vdeff

		svy_preg mlogit `doit' `*'

	/* Parse options. */

		local options "LOg TRace *"
		parse ", $S_VYeopt"
		if "`log'`trace'"=="" { local quiet "quietly" }
		local eopt "`trace' `options'"
		local options "MEFF MEFT *"
		parse ", $S_VYdopt"

	/* Ready weights for estimation. */

		if "$S_VYwgt"!="" {
			local awt "[aw=$S_VYexp]"
			local wt  "[iw=$S_VYexp]"
			local cwt "& ($S_VYexp)!=0"
		}

	/* Get mlogit estimates. */

		`quiet' di _n in gr "Computing weighted estimates ..."
		`quiet' mlogit $S_VYdepv $S_VYindv `awt' if $S_VYsub, `eopt'

		if _result(1)==0 | _result(1)==. {
			error 2000
		}
		qui count if $S_VYsub `cwt'  /* can't trust mlogit to count
		                                #obs
					     */
		local nobs = _result(1)
		matrix `b' = get(_b)
		matrix `V' = get(VCE)

	/* Transform `b' into a row vector. */

		TransB `b' `V'  /* delete this if mlogit is changed */

	/* Get row vector `C' containing categories of $S_VYdepv.
	   Set S_VYncat (number of categories) and S_VYbase (index of
	   base category).
	*/
		GetCat `b' `C'

	/* Use colnames of `b' for indepvars. */

		local dim  = colsof(`b')
		local nvar = `dim'/($S_VYncat - 1)
		local cons = colnumb(`b',"_cons")

		if `cons'!=. { local nvar = `nvar' - 1 }

		if `nvar' > 0 {
			matrix `A' = `b'[1,1..`nvar']
			global S_VYindv : colnames(`A')
		}
		else global S_VYindv /* empty macro */

	/* Get tempvars to hold scores. */

		local i 1
		while `i' <= $S_VYncat - 1 {
			tempvar tmp
			local scores "`scores' `tmp'"
			local i = `i' + 1
		}

	/* Compute scores. */

		Scores `b' `C' `scores'

	/* Change $S_VYbase from an index to value or a label. */

		SetBase `C'

	/* Unwind weight scaling of mlogit. */

		if "$S_VYwgt"!="" {
			tempvar sw
			qui gen double `sw' = sum($S_VYsub*($S_VYexp))
			scalar `s' = `nobs'/`sw'[_N]
			matrix `V' = `s'*`V'
		}

	/* Get mlogit estimates for meff. */

		if "`meff'`meft'"!="" {
			tempname Vmeff
			`quiet' di _n in gr "Computing unweighted estimates" /*
			*/ " for meff and meft ..."
			`quiet' mlogit $S_VYdepv $S_VYindv if $S_VYsub, `eopt'
			if _result(1)!=`nobs' {
				matrix `Vmeff' = J(`dim',`dim',0)
			}
			else {
				matrix `Vmeff' = get(VCE)
				local dimMeff = colsof(`Vmeff')
				if `dimMeff'!=`dim' {
					matrix `Vmeff' = J(`dim',`dim',0)
				}
			}
		}

	/* Compute variance estimate. */

		_robust `scores' `wt' if `doit', v(`V') vsrs(`Vdeff') /*
		*/	$S_VYopt zeroweight

	/* Post results. */

		local df = _result(3) - _result(2)

		matrix post `b' `V', dof(`df') depn($S_VYdepv)

	/* Set macros, etc. */

		local doption "$S_VYdopt" /* save display options */

		svy_preg save `Vdeff' `Vmeff' /* macros, matrices, F test */

		global S_E_cmd "svymlog"
	}
	nobreak {
		if _rc {
			local rc = _rc
			macro drop S_VY*
			discard
			exit `rc'
		}
	}

	svy_dreg, `doption'  /* display results */
end

program define Scores /* depvar markvar b catmatrix <variables for scores> */
	local b    "`1'"
	local C    "`2'"
	macro shift 2

	local j 1
	local i 1
	while `i' <= $S_VYncat {
		if `i' != $S_VYbase {
			local catval = `C'[`i',1]
			capture confirm integer number `catval'
			if _rc {
				di in red "`y' must have only integer values"
				exit 410
			}
			qui predict double ``j'' if $S_VYsub, outcome(`catval')
			qui replace ``j'' = cond($S_VYsub, /*
			*/	            ($S_VYdepv==`catval')-``j'', 0)
			local j = `j' + 1
		}
		local i = `i' + 1
	}
end

program define GetCat
	local b "`1'"
	local C "`2'"

	tempname A

/* Get categories.  This code assumes that mlogit does not drop any obs
   (except for missing values, which have already been accounted for).
*/
	qui tab $S_VYdepv if $S_VYsub, matrow(`C')
	global S_VYncat = _result(2)
	local nvar = colsof(`b')/($S_VYncat - 1)

/* Find base category. */

	local lblname : value label $S_VYdepv

	global S_VYbase $S_VYncat
	local i 1
	while `i' <= $S_VYncat - 1 {
		local j = `nvar'*`i'
		matrix `A' = `b'[1,`j']
		local cat : coleq(`A')
		capture confirm number `cat'
		if _rc {
			capture local cat = ("`cat'":`lblname')
			if _rc {
				di in red "error translating " /*
				*/ "category labels into "     /*
				*/ "numeric values"
				exit 182
			}
		}
		else local cat "`cat'"

		if reldif(`cat',`C'[`i',1]) > 1e-7 { /* found base category */
			global S_VYbase `i'
			exit
		}
		local i = `i' + 1
	}
end

program define SetBase
	local C "`1'"
	local baseval = `C'[$S_VYbase,1]

	local lblname : value label $S_VYdepv

	if "`lblname'"=="" { global S_VYbase `baseval'  }
	else		     global S_VYbase : label `lblname' `baseval'
end

program define TransB  /* delete this program if mlogit is changed
                          to give b as a row vector.
		       */
	local b "`1'"
	local V "`2'"
	tempname row newb s
	mat `newb' = `b'[1,.]
	local nr = rowsof(`b')
	local i 2
	while `i' <= `nr' {
		matrix `row' = `b'[`i',.]
		matrix `newb' = `newb' , `row'
		local i = `i' + 1
	}
	matrix `b' = `V'[1,.] /* steal labels from `V' */
	scalar `s' = 0
	matrix `b' = `s'*`b'
	matrix `b' = `newb' + `b' /* this uses labels from RHS `b' */
end
